@@ -60,7 +60,7 @@ except ImportError:
         return _convert(node_or_string)
 
 
-__version__ = '1.0'
+__version__ = '1.1'
 __all__ = ['Client', 'Model', 'Record', 'RecordList', 'Service', 'read_config']
 
 CONF_FILE = 'erppeek.ini'
@@ -363,26 +363,27 @@ class Client(object):
 
         If the `password` is not available, it will be asked.
         """
+        previous_db = self._db
         if database:
             dbs = self.db.list()
             if database not in dbs:
                 print("Error: Database '%s' does not exist: %s" %
                       (database, dbs))
                 return
-            if not self._db:
-                self._db = database
-        elif not self._db:
+            self._db = database
+        elif not previous_db:
             print('Error: Not connected')
             return
         (uid, password) = self._auth(user, password)
-        if uid is False:
+        if uid:
+            self.user = user
+            if database and previous_db != database:
+                self._environment = None
+        else:
+            if previous_db:
+                self._db = previous_db
             print('Error: Invalid username or password')
             return
-        self.user = user
-        if database:
-            if self._db != database:
-                self._environment = None
-            self._db = database
 
         # Authenticated endpoints
         def authenticated(method):
@@ -478,9 +479,9 @@ class Client(object):
                 g['do'] = None
                 g.update(dict.fromkeys(global_names))
 
-        def login(self, user, database=None):
+        def login(self, user, password=None, database=None):
             """Switch `user` and (optionally) `database`."""
-            if self._login(user, database=database):
+            if self._login(user, password=password, database=database):
                 # Register the new globals()
                 self.connect()
 
@@ -504,10 +505,8 @@ class Client(object):
                 progress, users = self.db.get_progress(passwd, thread_id)
             # [1.0, [{'login': 'admin', 'password': 'admin',
             #         'name': 'Administrator'}]]
-            self._login(users[0]['login'], users[0]['password'],
-                        database=database)
-            if self.connect is not None:
-                self.connect()
+            self.login(users[0]['login'], users[0]['password'],
+                       database=database)
         except KeyboardInterrupt:
             print({'id': thread_id, 'progress': progress})
 
@@ -519,15 +518,19 @@ class Client(object):
         Method `params` are allowed.  If needed, keyword
         arguments are collected in `kwargs`.
         """
-        assert isinstance(obj, basestring) and isinstance(method, basestring)
+        assert isinstance(obj, basestring)
+        assert isinstance(method, basestring) and method != 'browse'
         context = kwargs.pop('context', None)
+        ordered = None
         if method in ('read', 'name_get'):
             assert params
             if issearchdomain(params[0]):
                 # Combine search+read
                 search_params = searchargs(params[:1], kwargs, context)
+                ordered = len(search_params) > 3 and search_params[3]
                 ids = self._execute(obj, 'search', *search_params)
             else:
+                ordered = kwargs.pop('order', False)
                 ids = params[0]
             if not ids:
                 return []
@@ -552,7 +555,14 @@ class Client(object):
         # Ignore extra keyword arguments
         for item in kwargs.items():
             print('Ignoring: %s = %r' % item)
-        return self._execute(obj, method, *params)
+        res = self._execute(obj, method, *params)
+        if res and ordered:
+            # The results are not in the same order as the ids
+            # when received from the server
+            assert len(res) == len(set(ids))
+            resdic = dict([(val['id'], val) for val in res])
+            res = [resdic[id_] for id_ in ids]
+        return res
 
     def exec_workflow(self, obj, signal, obj_id):
         """Wrapper around ``object.exec_workflow`` RPC method.
@@ -664,6 +674,11 @@ class Client(object):
 
         If `domain` is not a single id, the returned value is a list of items.
         Each item complies with the rules of the previous paragraph.
+
+        The optional keyword arguments `offset`, `limit` and `order` are
+        used to restrict the search.  The `order` is also used to order the
+        results returned.  Note: the low-level RPC method ``read`` itself does
+        not preserve the order of the results.
         """
         fmt = None
         if len(params) > 1 and isinstance(params[1], basestring):
@@ -966,25 +981,27 @@ class RecordList(object):
         if context is None and self._context:
             context = self._context
 
+        if not self._ids:
+            return []
+
         client = self._model.client
         values = client.read(self._model._name, self._ids,
-                             fields, context=context)
+                             fields, order=True, context=context)
 
-        if not values:
-            return values
+        if values:
+            if isinstance(values[0], dict):
+                browse_values = self._model._browse_values
+                return [browse_values(v) for v in values]
 
-        if isinstance(values[0], dict):
-            browse_values = self._model._browse_values
-            return [browse_values(v) for v in values]
+            if '%(' not in fields:
+                field = self._model._fields[fields]
 
-        field = self._model._fields[fields]
-
-        if field['type'] == 'many2one':
-            rel_model = client.model(field['relation'])
-            return RecordList(rel_model, values, context=context)
-        if field['type'] in ('one2many', 'many2many'):
-            rel_model = client.model(field['relation'])
-            return [RecordList(rel_model, v) for v in values]
+                if field['type'] == 'many2one':
+                    rel_model = client.model(field['relation'])
+                    return RecordList(rel_model, values, context=context)
+                if field['type'] in ('one2many', 'many2many'):
+                    rel_model = client.model(field['relation'])
+                    return [RecordList(rel_model, v) for v in values]
         return values
 
     def __getitem__(self, key):
@@ -1089,6 +1106,8 @@ class Record(object):
                               fields, context=context)
         if isinstance(rv, dict):
             return self._update(rv)
+        elif '%(' in fields:
+            return rv
         else:
             return self._update({fields: rv})[fields]
 
@@ -1154,8 +1173,13 @@ class Record(object):
             """Wrapper for client.execute(%r, %r, %d, *params, **kwargs)."""
             if context:
                 kwargs.setdefault('context', context)
-            return self.client.execute(
-                self._model_name, attr, self.id, *params, **kwargs)
+            res = self.client.execute(
+                self._model_name, attr, [self.id], *params, **kwargs)
+            try:
+                (res,) = res
+            except Exception:
+                pass
+            return res
         wrapper.__name__ = attr
         wrapper.__doc__ %= (self._model_name, attr, self.id)
         self.__dict__[attr] = mobj = wrapper.__get__(self, type(self))
@@ -1257,10 +1281,10 @@ def main():
         help='read connection settings from the given section')
     parser.add_option(
         '-c', '--config', default=CONF_FILE,
-        help='specify alternate config file (default %r)' % CONF_FILE)
+        help='specify alternate config file (default: %r)' % CONF_FILE)
     parser.add_option(
         '--server', default=DEFAULT_URL,
-        help='full URL to the XML-RPC server (default %s)' % DEFAULT_URL)
+        help='full URL to the XML-RPC server (default: %s)' % DEFAULT_URL)
     parser.add_option('-d', '--db', default=DEFAULT_DB, help='database')
     parser.add_option('-u', '--user', default=DEFAULT_USER, help='username')
     parser.add_option(
